package javax.swing.text.html;

import javax.swing.text.Style;
import javax.swing.text.Document;
import java.util.Enumeration;
import javax.swing.text.BadLocationException;
import java.io.IOException;
import javax.swing.text.AttributeSet;
import javax.swing.text.ElementIterator;
import javax.swing.text.StyleConstants;
import javax.swing.text.SimpleAttributeSet;
import java.io.Writer;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.Segment;
import java.util.Vector;
import javax.swing.text.Element;
import java.util.Stack;
import javax.swing.text.AbstractWriter;

public class HTMLWriter extends AbstractWriter
{
    private Stack<Element> blockElementStack;
    private boolean inContent;
    private boolean inPre;
    private int preEndOffset;
    private boolean inTextArea;
    private boolean newlineOutputed;
    private boolean completeDoc;
    private Vector<HTML.Tag> tags;
    private Vector<Object> tagValues;
    private Segment segment;
    private Vector<HTML.Tag> tagsToRemove;
    private boolean wroteHead;
    private boolean replaceEntities;
    private char[] tempChars;
    private boolean indentNext;
    private boolean writeCSS;
    private MutableAttributeSet convAttr;
    private MutableAttributeSet oConvAttr;
    private boolean indented;
    
    public HTMLWriter(Writer writer, HTMLDocument htmlDocument) {
        this(writer, htmlDocument, 0, htmlDocument.getLength());
    }
    
    public HTMLWriter(Writer w, HTMLDocument doc, int pos, int len) {
        super(w, doc, pos, len);
        this.blockElementStack = new Stack();
        this.inContent = false;
        this.inPre = false;
        this.inTextArea = false;
        this.newlineOutputed = false;
        this.tags = new Vector(10);
        this.tagValues = new Vector(10);
        this.tagsToRemove = new Vector(10);
        this.indentNext = false;
        this.writeCSS = false;
        this.convAttr = new SimpleAttributeSet();
        this.oConvAttr = new SimpleAttributeSet();
        this.indented = false;
        this.completeDoc = (pos == 0 && len == doc.getLength());
        this.setLineLength(80);
    }
    
    public void write() throws IOException, BadLocationException {
        ElementIterator elementIterator = this.getElementIterator();
        Element element = null;
        this.wroteHead = false;
        this.setCurrentLineLength(0);
        this.setCanWrapLines(this.replaceEntities = false);
        if (this.segment == null) {
            this.segment = new Segment();
        }
        this.inPre = false;
        boolean n = false;
        Element next;
        while ((next = elementIterator.next()) != null) {
            if (!this.inRange(next)) {
                if (!this.completeDoc || next.getAttributes().getAttribute(StyleConstants.NameAttribute) != HTML.Tag.BODY) {
                    continue;
                }
                n = true;
            }
            if (element != null) {
                if (this.indentNeedsIncrementing(element, next)) {
                    this.incrIndent();
                }
                else if (element.getParentElement() != next.getParentElement()) {
                    for (Element element3 = (Element)this.blockElementStack.peek(); element3 != next.getParentElement(); element3 = (Element)this.blockElementStack.peek()) {
                        this.blockElementStack.pop();
                        if (!this.synthesizedElement(element3)) {
                            AttributeSet attributes = element3.getAttributes();
                            if (!this.matchNameAttribute(attributes, HTML.Tag.PRE) && !this.isFormElementWithContent(attributes)) {
                                this.decrIndent();
                            }
                            this.endTag(element3);
                        }
                    }
                }
                else if (element.getParentElement() == next.getParentElement()) {
                    Element element2 = (Element)this.blockElementStack.peek();
                    if (element2 == element) {
                        this.blockElementStack.pop();
                        this.endTag(element2);
                    }
                }
            }
            if (!next.isLeaf() || this.isFormElementWithContent(next.getAttributes())) {
                this.blockElementStack.push(next);
                this.startTag(next);
            }
            else {
                this.emptyTag(next);
            }
            element = next;
        }
        this.closeOutUnwantedEmbeddedTags(null);
        if (!(n == 0)) {
            this.blockElementStack.pop();
            this.endTag(element);
        }
        while (!this.blockElementStack.empty()) {
            Element element4 = (Element)this.blockElementStack.pop();
            if (this.synthesizedElement(element4)) {
                continue;
            }
            AttributeSet attributes2 = element4.getAttributes();
            if (!this.matchNameAttribute(attributes2, HTML.Tag.PRE) && !this.isFormElementWithContent(attributes2)) {
                this.decrIndent();
            }
            this.endTag(element4);
        }
        if (this.completeDoc) {
            this.writeAdditionalComments();
        }
        this.segment.array = null;
    }
    
    @Override
    protected void writeAttributes(AttributeSet set) throws IOException {
        this.convAttr.removeAttributes(this.convAttr);
        convertToHTML32(set, this.convAttr);
        Enumeration<Object> attributeNames = this.convAttr.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if (nextElement instanceof HTML.Tag || nextElement instanceof StyleConstants) {
                continue;
            }
            if (nextElement == HTML.Attribute.ENDTAG) {
                continue;
            }
            super.write(" " + nextElement + "=\"" + this.convAttr.getAttribute(nextElement) + "\"");
        }
    }
    
    protected void emptyTag(Element element) throws BadLocationException, IOException {
        if (!this.inContent && !this.inPre) {
            this.indentSmart();
        }
        AttributeSet attributes = element.getAttributes();
        this.closeOutUnwantedEmbeddedTags(attributes);
        this.writeEmbeddedTags(attributes);
        if (this.matchNameAttribute(attributes, HTML.Tag.CONTENT)) {
            this.inContent = true;
            this.text(element);
        }
        else if (this.matchNameAttribute(attributes, HTML.Tag.COMMENT)) {
            this.comment(element);
        }
        else {
            int blockTag = this.isBlockTag(element.getAttributes());
            if (this.inContent && blockTag) {
                this.writeLineSeparator();
                this.indentSmart();
            }
            Object o = (attributes != null) ? attributes.getAttribute(StyleConstants.NameAttribute) : null;
            Object o2 = (attributes != null) ? attributes.getAttribute(HTML.Attribute.ENDTAG) : null;
            boolean n = false;
            if (o != null && o2 != null && o2 instanceof String && o2.equals("true")) {
                n = true;
            }
            if (this.completeDoc && this.matchNameAttribute(attributes, HTML.Tag.HEAD)) {
                if (!(n == 0)) {
                    this.writeStyles(((HTMLDocument)this.getDocument()).getStyleSheet());
                }
                this.wroteHead = true;
            }
            super.write('<');
            if (!(n == 0)) {
                super.write('/');
            }
            super.write(element.getName());
            this.writeAttributes(attributes);
            super.write('>');
            if (this.matchNameAttribute(attributes, HTML.Tag.TITLE) && n == 0) {
                super.write((String)element.getDocument().getProperty("title"));
            }
            else if (!this.inContent || blockTag) {
                this.writeLineSeparator();
                if (blockTag && this.inContent) {
                    this.indentSmart();
                }
            }
        }
    }
    
    protected boolean isBlockTag(AttributeSet set) {
        Object attribute = set.getAttribute(StyleConstants.NameAttribute);
        return attribute instanceof HTML.Tag && ((HTML.Tag)attribute).isBlock();
    }
    
    protected void startTag(Element element) throws IOException, BadLocationException {
        if (this.synthesizedElement(element)) {
            return;
        }
        AttributeSet attributes = element.getAttributes();
        Object attribute = attributes.getAttribute(StyleConstants.NameAttribute);
        HTML.Tag tag;
        if ((attribute instanceof HTML.Tag)) {
            tag = (HTML.Tag)attribute;
        }
        else {
            tag = null;
        }
        if (tag == HTML.Tag.PRE) {
            this.inPre = true;
            this.preEndOffset = element.getEndOffset();
        }
        this.closeOutUnwantedEmbeddedTags(attributes);
        if (this.inContent) {
            this.writeLineSeparator();
            this.inContent = false;
            this.newlineOutputed = false;
        }
        if (this.completeDoc && tag == HTML.Tag.BODY && !this.wroteHead) {
            this.wroteHead = true;
            this.indentSmart();
            super.write("<head>");
            this.writeLineSeparator();
            this.incrIndent();
            this.writeStyles(((HTMLDocument)this.getDocument()).getStyleSheet());
            this.decrIndent();
            this.writeLineSeparator();
            this.indentSmart();
            super.write("</head>");
            this.writeLineSeparator();
        }
        this.indentSmart();
        super.write('<');
        super.write(element.getName());
        this.writeAttributes(attributes);
        super.write('>');
        if (tag != HTML.Tag.PRE) {
            this.writeLineSeparator();
        }
        if (tag == HTML.Tag.TEXTAREA) {
            this.textAreaContent(element.getAttributes());
        }
        else if (tag == HTML.Tag.SELECT) {
            this.selectContent(element.getAttributes());
        }
        else if (this.completeDoc && tag == HTML.Tag.BODY) {
            this.writeMaps(((HTMLDocument)this.getDocument()).getMaps());
        }
        else if (tag == HTML.Tag.HEAD) {
            HTMLDocument htmlDocument = (HTMLDocument)this.getDocument();
            this.wroteHead = true;
            this.incrIndent();
            this.writeStyles(htmlDocument.getStyleSheet());
            if (htmlDocument.hasBaseTag()) {
                this.indentSmart();
                super.write("<base href=\"" + htmlDocument.getBase() + "\">");
                this.writeLineSeparator();
            }
            this.decrIndent();
        }
        return;
    }
    
    protected void textAreaContent(AttributeSet set) throws BadLocationException, IOException {
        Document document = (Document)set.getAttribute(StyleConstants.ModelAttribute);
        if (document != null && document.getLength() > 0) {
            if (this.segment == null) {
                this.segment = new Segment();
            }
            document.getText(0, document.getLength(), this.segment);
            if (this.segment.count > 0) {
                this.inTextArea = true;
                this.incrIndent();
                this.indentSmart();
                this.setCanWrapLines(true);
                this.replaceEntities = true;
                super.write(this.segment.array, this.segment.offset, this.segment.count);
                this.setCanWrapLines(this.replaceEntities = false);
                this.writeLineSeparator();
                this.inTextArea = false;
                this.decrIndent();
            }
        }
    }
    
    @Override
    protected void text(Element element) throws BadLocationException, IOException {
        int max = Math.max(this.getStartOffset(), element.getStartOffset());
        int min = Math.min(this.getEndOffset(), element.getEndOffset());
        if (max < min) {
            if (this.segment == null) {
                this.segment = new Segment();
            }
            this.getDocument().getText(max, min - max, this.segment);
            this.newlineOutputed = false;
            if (this.segment.count > 0) {
                if (this.segment.array[this.segment.offset + this.segment.count - 1] == '\n') {
                    this.newlineOutputed = true;
                }
                if (this.inPre && min == this.preEndOffset) {
                    if (this.segment.count > 1) {
                        Segment segment = this.segment;
                        --segment.count;
                    }
                    else {
                        return;
                    }
                }
                this.replaceEntities = true;
                this.setCanWrapLines(!this.inPre);
                super.write(this.segment.array, this.segment.offset, this.segment.count);
                this.setCanWrapLines(false);
                this.replaceEntities = false;
            }
        }
    }
    
    protected void selectContent(AttributeSet set) throws IOException {
        Object attribute = set.getAttribute(StyleConstants.ModelAttribute);
        this.incrIndent();
        if ((attribute instanceof OptionListModel)) {
            OptionListModel<Object> optionListModel = (OptionListModel)attribute;
            for (int size2 = optionListModel.getSize(), j = 0; j < size2; ++j) {
                this.writeOption((Option)optionListModel.getElementAt(j));
            }
        }
        else if ((attribute instanceof OptionComboBoxModel)) {
            OptionComboBoxModel<Object> optionComboBoxModel = (OptionComboBoxModel)attribute;
            for (int size = optionComboBoxModel.getSize(), i = 0; i < size; ++i) {
                this.writeOption((Option)optionComboBoxModel.getElementAt(i));
            }
        }
        this.decrIndent();
    }
    
    protected void writeOption(Option option) throws IOException {
        this.indentSmart();
        super.write('<');
        super.write("option");
        Object attribute = option.getAttributes().getAttribute(HTML.Attribute.VALUE);
        if (attribute != null) {
            super.write(" value=" + attribute);
        }
        if (option.isSelected()) {
            super.write(" selected");
        }
        super.write('>');
        if (option.getLabel() != null) {
            super.write(option.getLabel());
        }
        this.writeLineSeparator();
    }
    
    protected void endTag(Element element) throws IOException {
        if (this.synthesizedElement(element)) {
            return;
        }
        this.closeOutUnwantedEmbeddedTags(element.getAttributes());
        if (this.inContent) {
            if (!this.newlineOutputed && !this.inPre) {
                this.writeLineSeparator();
            }
            this.newlineOutputed = false;
            this.inContent = false;
        }
        if (!this.inPre) {
            this.indentSmart();
        }
        if (this.matchNameAttribute(element.getAttributes(), HTML.Tag.PRE)) {
            this.inPre = false;
        }
        super.write('<');
        super.write('/');
        super.write(element.getName());
        super.write('>');
        this.writeLineSeparator();
        return;
    }
    
    protected void comment(Element element) throws BadLocationException, IOException {
        AttributeSet attributes = element.getAttributes();
        if (this.matchNameAttribute(attributes, HTML.Tag.COMMENT)) {
            Object attribute = attributes.getAttribute(HTML.Attribute.COMMENT);
            if ((attribute instanceof String)) {
                this.writeComment((String)attribute);
            }
            else {
                this.writeComment(null);
            }
        }
    }
    
    void writeComment(String content) throws IOException {
        super.write("<!--");
        if (content != null) {
            super.write(content);
        }
        super.write("-->");
        this.writeLineSeparator();
        this.indentSmart();
    }
    
    void writeAdditionalComments() throws IOException {
        Object property = this.getDocument().getProperty("AdditionalComments");
        if ((property instanceof Vector)) {
            Vector<Object> vector = (Vector)property;
            for (int i = 0; i < vector.size(); ++i) {
                this.writeComment(vector.elementAt(i).toString());
            }
        }
    }
    
    protected boolean synthesizedElement(Element element) {
        return this.matchNameAttribute(element.getAttributes(), HTML.Tag.IMPLIED);
    }
    
    protected boolean matchNameAttribute(AttributeSet set, HTML.Tag tag) {
        Object attribute = set.getAttribute(StyleConstants.NameAttribute);
        return attribute instanceof HTML.Tag && (HTML.Tag)attribute == tag;
    }
    
    protected void writeEmbeddedTags(AttributeSet convertToHTML) throws IOException {
        convertToHTML = this.convertToHTML(convertToHTML, this.oConvAttr);
        Enumeration<Object> attributeNames = convertToHTML.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if ((nextElement instanceof HTML.Tag)) {
                HTML.Tag tag = (HTML.Tag)nextElement;
                if (tag == HTML.Tag.FORM) {
                    continue;
                }
                if (this.tags.contains(tag)) {
                    continue;
                }
                super.write('<');
                super.write(tag.toString());
                Object attribute = convertToHTML.getAttribute(tag);
                if (attribute != null && attribute instanceof AttributeSet) {
                    this.writeAttributes((AttributeSet)attribute);
                }
                super.write('>');
                this.tags.addElement(tag);
                this.tagValues.addElement(attribute);
            }
            else {
                continue;
            }
        }
    }
    
    private boolean noMatchForTagInAttributes(AttributeSet set, HTML.Tag tag, Object o) {
        if (set != null && set.isDefined(tag)) {
            Object attribute = set.getAttribute(tag);
            if ((o != null) ? (attribute == null || !o.equals(attribute)) : (attribute != null)) {}
            return false;
        }
        return true;
    }
    
    protected void closeOutUnwantedEmbeddedTags(AttributeSet convertToHTML) throws IOException {
        this.tagsToRemove.removeAllElements();
        convertToHTML = this.convertToHTML(convertToHTML, null);
        int i = -1;
        int size = this.tags.size();
        for (int j = size - 1; j >= 0; --j) {
            HTML.Tag obj = (HTML.Tag)this.tags.elementAt(j);
            Object element = this.tagValues.elementAt(j);
            if (convertToHTML == null || this.noMatchForTagInAttributes(convertToHTML, obj, element)) {
                i = j;
                this.tagsToRemove.addElement(obj);
            }
        }
        if (i != -1) {
            int n = size - i == this.tagsToRemove.size();
            for (int k = size - 1; k >= i; --k) {
                HTML.Tag o = (HTML.Tag)this.tags.elementAt(k);
                if (n || this.tagsToRemove.contains(o)) {
                    this.tags.removeElementAt(k);
                    this.tagValues.removeElementAt(k);
                }
                super.write('<');
                super.write('/');
                super.write(o.toString());
                super.write('>');
            }
            int size2;
            for (size2 = this.tags.size(), ; i < size2; ++i) {
                HTML.Tag tag = (HTML.Tag)this.tags.elementAt(i);
                super.write('<');
                super.write(tag.toString());
                Object element2 = this.tagValues.elementAt(i);
                if (element2 != null && element2 instanceof AttributeSet) {
                    this.writeAttributes((AttributeSet)element2);
                }
                super.write('>');
            }
        }
    }
    
    private boolean isFormElementWithContent(AttributeSet set) {
        return this.matchNameAttribute(set, HTML.Tag.TEXTAREA) || this.matchNameAttribute(set, HTML.Tag.SELECT);
    }
    
    private boolean indentNeedsIncrementing(Element element, Element element2) {
        if (element2.getParentElement() == element && !this.inPre) {
            if (this.indentNext) {
                this.indentNext = false;
                return true;
            }
            if (this.synthesizedElement(element2)) {
                this.indentNext = true;
            }
            else if (!this.synthesizedElement(element)) {
                return true;
            }
        }
        return false;
    }
    
    void writeMaps(Enumeration enumeration) throws IOException {
        if (enumeration != null) {
            while (enumeration.hasMoreElements()) {
                Map map = (Map)enumeration.nextElement();
                String name = map.getName();
                this.incrIndent();
                this.indentSmart();
                super.write("<map");
                if (name != null) {
                    super.write(" name=\"");
                    super.write(name);
                    super.write("\">");
                }
                else {
                    super.write('>');
                }
                this.writeLineSeparator();
                this.incrIndent();
                AttributeSet[] areas = map.getAreas();
                if (areas != null) {
                    for (int i = 0; i < areas.length; ++i) {
                        this.indentSmart();
                        super.write("<area");
                        this.writeAttributes(areas[i]);
                        super.write("></area>");
                        this.writeLineSeparator();
                    }
                }
                this.decrIndent();
                this.indentSmart();
                super.write("</map>");
                this.writeLineSeparator();
                this.decrIndent();
            }
        }
    }
    
    void writeStyles(StyleSheet styleSheet) throws IOException {
        if (styleSheet != null) {
            Enumeration<Object> styleNames = styleSheet.getStyleNames();
            if (styleNames != null) {
                boolean n = false;
                while (styleNames.hasMoreElements()) {
                    String s = (String)styleNames.nextElement();
                    if (!"default".equals(s) && this.writeStyle(s, styleSheet.getStyle(s), (boolean)n)) {
                        n = true;
                    }
                    else {
                        continue;
                    }
                }
                if (!(n == 0)) {
                    this.writeStyleEndTag();
                }
            }
        }
    }
    
    boolean writeStyle(String content, Style style, boolean b) throws IOException {
        boolean n = false;
        Enumeration<Object> attributeNames = style.getAttributeNames();
        if (attributeNames != null) {
            while (attributeNames.hasMoreElements()) {
                Object nextElement = attributeNames.nextElement();
                if ((nextElement instanceof CSS.Attribute)) {
                    String string = style.getAttribute(nextElement).toString();
                    if (string != null) {
                        if (!b) {
                            this.writeStyleStartTag();
                            b = true;
                        }
                        if (n == 0) {
                            n = true;
                            this.indentSmart();
                            super.write(content);
                            super.write(" {");
                        }
                        else {
                            super.write(";");
                        }
                        super.write(' ');
                        super.write(nextElement.toString());
                        super.write(": ");
                        super.write(string);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
        }
        if (!(n == 0)) {
            super.write(" }");
            this.writeLineSeparator();
        }
        return n;
    }
    
    void writeStyleStartTag() throws IOException {
        this.indentSmart();
        super.write("<style type=\"text/css\">");
        this.incrIndent();
        this.writeLineSeparator();
        this.indentSmart();
        super.write("<!--");
        this.incrIndent();
        this.writeLineSeparator();
    }
    
    void writeStyleEndTag() throws IOException {
        this.decrIndent();
        this.indentSmart();
        super.write("-->");
        this.writeLineSeparator();
        this.decrIndent();
        this.indentSmart();
        super.write("</style>");
        this.writeLineSeparator();
        this.indentSmart();
    }
    
    AttributeSet convertToHTML(AttributeSet set, MutableAttributeSet convAttr) {
        if (convAttr == null) {
            convAttr = this.convAttr;
        }
        convAttr.removeAttributes(convAttr);
        if (this.writeCSS) {
            convertToHTML40(set, convAttr);
            return convAttr;
        }
        convertToHTML32(set, convAttr);
        return convAttr;
    }
    
    private static void convertToHTML32(AttributeSet set, MutableAttributeSet set2) {
        if (set == null) {
            return;
        }
        Enumeration<Object> attributeNames = set.getAttributeNames();
        String s = "";
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if ((nextElement instanceof CSS.Attribute)) {
                if (nextElement == CSS.Attribute.FONT_FAMILY || nextElement == CSS.Attribute.FONT_SIZE || nextElement == CSS.Attribute.COLOR) {
                    createFontAttribute((CSS.Attribute)nextElement, set, set2);
                }
                else if (nextElement == CSS.Attribute.FONT_WEIGHT) {
                    CSS.FontWeight fontWeight = (CSS.FontWeight)set.getAttribute(CSS.Attribute.FONT_WEIGHT);
                    if (fontWeight != null && fontWeight.getValue() > 400) {
                        addAttribute(set2, HTML.Tag.B, SimpleAttributeSet.EMPTY);
                    }
                    else {
                        continue;
                    }
                }
                else if (nextElement == CSS.Attribute.FONT_STYLE) {
                    if (set.getAttribute(nextElement).toString().indexOf("italic") >= 0) {
                        addAttribute(set2, HTML.Tag.I, SimpleAttributeSet.EMPTY);
                    }
                    else {
                        continue;
                    }
                }
                else if (nextElement == CSS.Attribute.TEXT_DECORATION) {
                    String string2 = set.getAttribute(nextElement).toString();
                    if (string2.indexOf("underline") >= 0) {
                        addAttribute(set2, HTML.Tag.U, SimpleAttributeSet.EMPTY);
                    }
                    if (string2.indexOf("line-through") >= 0) {
                        addAttribute(set2, HTML.Tag.STRIKE, SimpleAttributeSet.EMPTY);
                    }
                    else {
                        continue;
                    }
                }
                else if (nextElement == CSS.Attribute.VERTICAL_ALIGN) {
                    String string = set.getAttribute(nextElement).toString();
                    if (string.indexOf("sup") >= 0) {
                        addAttribute(set2, HTML.Tag.SUP, SimpleAttributeSet.EMPTY);
                    }
                    if (string.indexOf("sub") >= 0) {
                        addAttribute(set2, HTML.Tag.SUB, SimpleAttributeSet.EMPTY);
                    }
                    else {
                        continue;
                    }
                }
                else if (nextElement == CSS.Attribute.TEXT_ALIGN) {
                    addAttribute(set2, HTML.Attribute.ALIGN, set.getAttribute(nextElement).toString());
                }
                else {
                    if (s.length() > 0) {
                        s += "; ";
                    }
                    s = s + nextElement + ": " + set.getAttribute(nextElement);
                }
            }
            else {
                Object o = set.getAttribute(nextElement);
                if ((o instanceof AttributeSet)) {
                    o = ((AttributeSet)o).copyAttributes();
                }
                addAttribute(set2, nextElement, o);
            }
        }
        if (s.length() > 0) {
            set2.addAttribute(HTML.Attribute.STYLE, s);
        }
        return;
    }
    
    private static void addAttribute(MutableAttributeSet set, Object o, Object o2) {
        Object attribute = set.getAttribute(o);
        if (attribute == null || attribute == SimpleAttributeSet.EMPTY) {
            set.addAttribute(o, o2);
        }
        else if (attribute instanceof MutableAttributeSet && o2 instanceof AttributeSet) {
            ((MutableAttributeSet)attribute).addAttributes((AttributeSet)o2);
        }
    }
    
    private static void createFontAttribute(CSS.Attribute attribute, AttributeSet set, MutableAttributeSet set2) {
        MutableAttributeSet set3 = (MutableAttributeSet)set2.getAttribute(HTML.Tag.FONT);
        if (set3 == null) {
            set3 = new SimpleAttributeSet();
            set2.addAttribute(HTML.Tag.FONT, set3);
        }
        String string = set.getAttribute(attribute).toString();
        if (attribute == CSS.Attribute.FONT_FAMILY) {
            set3.addAttribute(HTML.Attribute.FACE, string);
        }
        else if (attribute == CSS.Attribute.FONT_SIZE) {
            set3.addAttribute(HTML.Attribute.SIZE, string);
        }
        else if (attribute == CSS.Attribute.COLOR) {
            set3.addAttribute(HTML.Attribute.COLOR, string);
        }
    }
    
    private static void convertToHTML40(AttributeSet set, MutableAttributeSet set2) {
        Enumeration<Object> attributeNames = set.getAttributeNames();
        String string = "";
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if ((nextElement instanceof CSS.Attribute)) {
                string = string + " " + nextElement + "=" + set.getAttribute(nextElement) + ";";
            }
            else {
                set2.addAttribute(nextElement, set.getAttribute(nextElement));
            }
        }
        if (string.length() > 0) {
            set2.addAttribute(HTML.Attribute.STYLE, string);
        }
    }
    
    @Override
    protected void writeLineSeparator() throws IOException {
        int replaceEntities = this.replaceEntities;
        this.replaceEntities = false;
        super.writeLineSeparator();
        this.replaceEntities = replaceEntities;
        this.indented = false;
    }
    
    @Override
    protected void output(char[] content, int i, int length) throws IOException {
        if (!this.replaceEntities) {
            super.output(content, i, length);
            return;
        }
        for (, length += i, ; i < length; ++i) {
            switch (content[i]) {
                case '<': {
                    if (i > i) {
                        super.output(content, i, i - i);
                    }
                    ++i;
                    this.output("&lt;");
                    break;
                }
                case '>': {
                    if (i > i) {
                        super.output(content, i, i - i);
                    }
                    ++i;
                    this.output("&gt;");
                    break;
                }
                case '&': {
                    if (i > i) {
                        super.output(content, i, i - i);
                    }
                    ++i;
                    this.output("&amp;");
                    break;
                }
                case '\"': {
                    if (i > i) {
                        super.output(content, i, i - i);
                    }
                    ++i;
                    this.output("&quot;");
                    break;
                }
                case '\t':
                case '\n':
                case '\r': {
                    break;
                }
                default: {
                    if (content[i] < ' ' || content[i] > '\u007f') {
                        if (i > i) {
                            super.output(content, i, i - i);
                        }
                        ++i;
                        this.output("&#");
                        this.output(String.valueOf((int)content[i]));
                        this.output(";");
                        break;
                    }
                    else {
                        break;
                    }
                }
            }
        }
        if (i < length) {
            super.output(content, i, length - i);
        }
        return;
    }
    
    private void output(String s) throws IOException {
        int length = s.length();
        if (this.tempChars == null || this.tempChars.length < length) {
            this.tempChars = new char[length];
        }
        s.getChars(0, length, this.tempChars, 0);
        super.output(this.tempChars, 0, length);
    }
    
    private void indentSmart() throws IOException {
        if (!this.indented) {
            this.indent();
            this.indented = true;
        }
    }
}
